#!/usr/bin/python
#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Stephan Hermann
# Copyright (C) 2008 Siegfried-A. Gevatter
# Copyright (C) 2008 Markus Korn
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

import sys
import os, os.path
import logging
from optparse import OptionParser

from leonov.backend import config, Global

def main():
    parser = OptionParser()
    parser.add_option("-f", "--frontend", dest="frontend", help="lets you choose which frontend to use; FRONTEND can be either  KDE or GTK.", metavar="FRONTEND")
    parser.add_option("-d", "--debug", action="store_true", dest="debug", help="enabled verbose mode.", default=False)
    (options, args) = parser.parse_args()
    
    if options.debug:
        log_level = logging.DEBUG
    else:
        log_level = logging.WARNING
    logging.basicConfig(format='%(asctime)s %(name)8s %(levelname)8s %(message)s', level=log_level)
    
    if not options.frontend:
        if config.get_frontend():
            options.frontend = config.get_frontend()
        else:
            logging.getLogger("leonov.main").debug("options.frontend = %s", options.frontend)
            options.frontend = "KDE"
            if os.environ.has_key("DESKTOP_SESSION"):
                desktop_session = os.environ["DESKTOP_SESSION"].lower()
                if "gnome" in desktop_session or "xfce" in desktop_session:
                    options.frontend = "GTK"
                if "kde4" in desktop_session:
                    options.frontend = "KDE"
    
    if options.frontend.upper() == "KDE":
        try:
            from leonov.kde import frontend
        except ImportError, e:
            if options.debug:
                raise
            else:
                print "Import Error: %s. Use --debug for more details." % e
                sys.exit(1)
    elif options.frontend.upper() == "GTK":
        try:
            from leonov.gtk import frontend
        except ImportError, e:
            if options.debug:
                raise
            else:
                print "Import Error: %s. Use --debug for more details." % e
                sys.exit(1)
    
    if not config.get_stay_authenticated():
        # here needs to be the check for auth_once_per_day
        if Global.cookie_file and os.path.isfile(Global.cookie_file):
            os.remove(Global.cookie_file)
    
    f = frontend()

if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        sys.exit(1)
