#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Siegfried-A. Gevatter
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

import xdg.BaseDirectory as basedir
import threading
import os

class Holder:
    
    def __repr__(self):
        return str(vars(self))
    
    def __str__(self):
        return self.__repr__()

class Progress:
    
    NAME_REG = "registered_thread"
    NAME_FIN = "finalized_thread"
    
    def __init__(self):
        self._data = []
    
    def _keys(self):
        return [i[0] for i in self._data]
    
    def _values(self):
        return [i[1] for i in self._data]
    
    def _index(self, key):
        return self._keys().index(key)
    
    def get_fraction(self, key=None):
        if key:
            return self._data[self._index(key)][1]
        else:
            return sum(self._values()) / len(self._data)
    
    def register(self, name=None, amount=1):
        for i in xrange(amount):
            if name:
                # Register an initialized thread
                if self.NAME_REG in self._keys():
                    # Replace an uninitialized thread, if present
                    del(self._data[self._index(self.NAME_REG)])
                self._data.append([name, 0])
            else:
                # Register an uninitialized thread
                self._data.append([self.NAME_REG, 0])
    
    def finalize(self, name=None):
        if name:
            if self.get_fraction(name) != 0:
                self._data.append([self.NAME_FIN, self.get_fraction(name)])
            del(self._data[self._index(name)])
        else:
            del(self._data[self._index(self.NAME_REG)])
    
    def update(self, name, fraction):
        if fraction > 1:
            # The size of data that needs to be retrieved is inaccurate
            fraction = 1
        self._data[self._index(name)][1] = fraction
    
    def clear(self):
        while self.NAME_FIN in self._keys():
            del(self._data[self._index(self.NAME_FIN)])

Global = Holder()

# TODO: Remove this as it is duplicated in leonov/backend/lpbackend.py
Global.cookie_file = basedir.load_first_config("leonov", "lp_cookie.txt")
if not Global.cookie_file:
    Global.cookie_file = os.path.join(basedir.save_config_path("leonov"),
        "lp_cookie.txt")

Global.progress = Progress()
Global.thread = threading.local()
