#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Stephan Hermann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################
import xdg.BaseDirectory as basedir
from pysqlite2 import dbapi2 as sqlite
import logging
import os.path

class LPDB():
    
    def __init__(self):
        self.dbname="launchpad.sql.db"
        self.savepath=os.path.join(basedir.save_config_path("leonov"),self.dbname)
        # check if that exists:
        if not os.path.exists(self.savepath):
            self.open_db()
            # create the tables:
            self.create_tables()
        else:
            self.open_db()
    def open_db(self):
        self.db_connection=sqlite.connect(self.savepath)
        self.cursor=self.db_connection.cursor()

    def create_tables(self):
        # Usertable
        self.create_table("lp_users","user_id INTEGER PRIMARY KEY AUTOINCREMENT,realname varchar(100),lp_id varchar(50)")
        
        # Project table
        self.create_table("lp_projects","project_id INTEGER PRIMARY KEY AUTOINCREMENT,name varchar(100),url varchar(255),registrar INTEGER,license_reviewed INTEGER,date varchar(30)")
        
    def create_table(self,table_name,table_data):
        # Usertable
        self.cursor.execute( "CREATE TABLE %s ( %s )" % (table_name,table_data))
        
    def select_from_table(self,table,select_string,filter_string=""):
        logging.getLogger("leonov.backend.sqlinterface").debug("LPDB::select_from_table: table %s select_string: %s filter_string: %s",table,select_string,filter_string)
        if table:
            if filter_string=="":
                self.cursor.execute("SELECT %s FROM %s" % (select_string,table))
            else:
                self.cursor.execute("SELECT %s FROM %s WHERE %s" % (select_string,table,filter_string))
        return self.cursor.fetchall()
        
class LPDB_Users(LPDB):
    def __init__(self):
        LPDB.__init__(self)
        
    def get_lp_users(self,select_filter=None,search_filter=None):
        logging.getLogger("leonov.backend.sqlinterface").debug("LPDB_Users::get_lp_users")
        if not select_filter:
            select_filter="*"
        if not search_filter:
            search_filter=""
        return self.select_from_table("lp_users",select_filter,search_filter)
        
    def add_lp_users(self,nick,realname):
        logging.getLogger("leonov.backend.sqlinterface").debug("LPDB_Users::add_lp_users")
        self.cursor.execute("INSERT INTO lp_users values(?,?,?)",(0,nick,realname))

    