#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Nathan Samson
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

import gtk
import os.path

class BugInfoWidgetComments(gtk.ScrolledWindow):
    
    def __init__(self, builder):
        gtk.ScrolledWindow.__init__(self)
        self._builder = builder
        self._builder.connect_signals(self)
        
        self._builder.get_object("showAllAttachmentsToggleButton") \
            .connect('toggled', self._onShowAllAttachmentsToggled)
    
        commentsVPaned = self._builder.get_object("commentsVPaned")
        commentsVPaned.unparent()
        self.add_with_viewport(commentsVPaned)
        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        
        commentsTreeView = self._builder.get_object("commentsTreeView")
        self._commentsModel = gtk.ListStore(int, str, str, str, object)
        commentsTreeView.set_model(self._commentsModel)
        
        col = gtk.TreeViewColumn('No', gtk.CellRendererText(), text=0)
        col1 = gtk.TreeViewColumn('Commentor', gtk.CellRendererText(), text=1)
        col2 = gtk.TreeViewColumn('Subject', gtk.CellRendererText(), text=2)
        col3 = gtk.TreeViewColumn('Date', gtk.CellRendererText(), text=3)
        commentsTreeView.append_column(col)
        commentsTreeView.append_column(col1)
        commentsTreeView.append_column(col2)
        commentsTreeView.append_column(col3)
        commentsTreeView.get_selection() \
                        .connect('changed', self._onCommentSelectionChanged)
        
        attachmentsTreeView = self._builder.get_object("attachmentsTreeView")
        self._attachmentsModel = gtk.ListStore(str, object)
        attachmentsTreeView.set_model(self._attachmentsModel)
        attachmentsTreeView.get_selection() \
                        .connect('changed', self._onAttachmentSelectionChanged)
        
        col = gtk.TreeViewColumn('Filename', gtk.CellRendererText(), text=0)
        attachmentsTreeView.append_column(col)
        self._attachmentsTreeView = attachmentsTreeView
        self._bug = None
        self._builder.get_object("saveAttachmentButton").set_sensitive(False)
        self._builder.get_object("saveAllAttachmentsButton").set_sensitive(False)
        self._builder.get_object("saveAttachmentButton").connect('clicked', self._onSaveAttachment)
        self._builder.get_object("saveAllAttachmentsButton").connect('clicked', self._onSaveAllAttachments)
        self._defaultDownloadPath = os.path.expanduser("~")
    
    def showBug(self, bug = None):
        self._bug = bug
        if not bug:
            return
        self._attachmentsModel.clear()
        self._commentsModel.clear()
        i = 1
        for comment in bug.comments:
            self._addComment(i, comment)
            i += 1
        if self._getShowAllAttachments():
            self._builder.get_object("saveAllAttachmentsButton").set_sensitive(True)
            self._showAttachments(bug.attachments)
        else:
            self._builder.get_object("saveAllAttachmentsButton").set_sensitive(False)
    
    def _onShowAllAttachmentsToggled(self, object):
        if not self._bug:
            return
        if object.get_active():
            self._builder.get_object("saveAllAttachmentsButton").set_sensitive(True)
            self._showAttachments(self._bug.attachments)
        else:
            self._builder.get_object("saveAllAttachmentsButton").set_sensitive(False)
            self._onCommentSelectionChanged(self._builder \
                                            .get_object("commentsTreeView") \
                                            .get_selection())
    
    def _onCommentSelectionChanged(self, selection):
        (model, iter) = selection.get_selected()
        if iter:
            self._showComment(self._commentsModel.get_value(iter, 4),
                              self._commentsModel.get_value(iter, 0)-1)
        else:
            if not self._getShowAllAttachments():
                self._attachmentsModel.clear()
    
    def _showComment(self, comment, counter):
        bf = gtk.TextBuffer()
        bf.set_text(comment.text)
        self._builder.get_object("commentTextView").set_buffer(bf)
        
        if not self._getShowAllAttachments():
            self._showAttachments(comment.attachments)
    
    def _addComment(self, counter, comment):
        if comment.user:
            username = comment.user.realname
        else:
            username = ""
        if comment.subject:
            subject = comment.subject
        else:
            subject = ""
        if comment.date:
            date = str(comment.date)
        else:
            date = ""
        self._commentsModel.append([counter, username, subject, date, comment])

    def _showAttachments(self, attachments):
        self._attachmentsModel.clear()
        for attachment in attachments:
            self._addAttachment(attachment)
    
    def _addAttachment(self, attachment):
        self._attachmentsModel.append([attachment.lp_filename, attachment])

    def _onAttachmentSelectionChanged(self, selection):
        (model, iter) = selection.get_selected()
        if iter:
            self._builder.get_object("saveAttachmentButton").set_sensitive(True)
        else:
            self._builder.get_object("saveAttachmentButton").set_sensitive(False)

    def _onSaveAttachment(self, obj):
        (model, iter) = self._attachmentsTreeView.get_selection().get_selected()
        if iter:
            attachment = model.get_value(iter, 1)
            path = self._getDownloadFile(attachment)
            if path == None:
                return # User cancelled the operation
            attachment.download(path)
    
    def _onSaveAllAttachments(self, obj):
        path = self._getDownloadPath()
        for attachment in self._bug.attachments:
            attachment.download(path+"/"+attachment.lp_filename)
    
    def _getDownloadFile(self, attachment):
        dlg = gtk.FileChooserDialog("Save attachment", None, 
                                    gtk.FILE_CHOOSER_ACTION_SAVE,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        dlg.set_do_overwrite_confirmation(True)
        dlg.set_current_folder(self._getDefaultDownloadPath())
        dlg.set_current_name(attachment.lp_filename)
        rsp = dlg.run()
        if rsp == gtk.RESPONSE_OK:
            self._setDefaultDownloadPath(dlg.get_current_folder())
            fileName = dlg.get_filename()
        else:
            fileName = None
        dlg.destroy()
        return fileName
    
    def _getDownloadPath(self):
        dlg = gtk.FileChooserDialog("Save attachments", None, 
                                    gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        dlg.set_current_folder(self._getDefaultDownloadPath())
        rsp = dlg.run()
        if rsp == gtk.RESPONSE_OK:
            self._setDefaultDownloadPath(dlg.get_current_folder())
            fileName = dlg.get_current_folder()
        else:
            fileName = None
        dlg.destroy()
        return fileName
    
    def _getDefaultDownloadPath(self):
        return self._defaultDownloadPath
    
    def _setDefaultDownloadPath(self, dir):
        self._defaultDownloadPath = dir
    
    def _getShowAllAttachments(self):
        return self._builder.get_object("showAllAttachmentsToggleButton") \
                   .get_active()
