#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Nathan Samson
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

import gobject
import gtk

class BugInfoWidgetGeneral(gtk.ScrolledWindow):

    __gsignals__ = {"show-reporters-task-list" : (gobject.SIGNAL_RUN_FIRST, 
                                                  gobject.TYPE_NONE,
                                                  (gobject.TYPE_STRING, ))}

    def __init__(self, builder):
        gtk.ScrolledWindow.__init__(self)
        self._builder = builder

        self._builder.get_object("showReportersTaskListButton") \
            .connect("clicked", self._onShowReportersTaskListButtonClicked)

        bugInfoTable = self._builder.get_object("bugInfoGeneralVBox")
        bugInfoTable.unparent()
        self.add_with_viewport(bugInfoTable)
        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        bugInfoTaskListTreeview = self._builder.get_object("bugInfoTaskListTreeview")
        self._taskListModel = gtk.ListStore(str, str, str, str, str, str)
        bugInfoTaskListTreeview.set_model(self._taskListModel)
        self._bugInfoTaskListTreeview = bugInfoTaskListTreeview
        
        self._newBugInfoTaskListTreeviewColumn('Task', 0)
        self._newBugInfoTaskListTreeviewColumn('Project/Distro', 1)
        self._newBugInfoTaskListTreeviewColumn('Release', 2)
        self._newBugInfoTaskListTreeviewColumn('Status', 3)
        self._newBugInfoTaskListTreeviewColumn('Importance', 4)
        self._newBugInfoTaskListTreeviewColumn('Assignee', 5)
        self._taskListModel.set_sort_column_id(0, gtk.SORT_ASCENDING)
        self._bug = None

    def _onTaskColumnClicked(self, column, colID):
        if column.get_sort_order() == gtk.SORT_ASCENDING:
            sortOrder = gtk.SORT_DESCENDING
        else:
            sortOrder = gtk.SORT_ASCENDING
        column.set_sort_order(sortOrder)
        self._taskListModel.set_sort_column_id(colID, sortOrder)

    def showBug(self, bug = None):
        self._taskListModel.clear()
        self._bug = bug
        if bug:
            self._builder.get_object("showReportersTaskListButton").set_sensitive(True)
            self._builder.get_object("bugInfoSummaryEntry").set_text(bug.summary)
            self._builder.get_object("bugInfoReporterEntry").set_text(bug.reporter)
            descBuffer = gtk.TextBuffer()
            descBuffer.set_text(bug.description)
            self._builder.get_object("bugInfoDescriptionTextView").set_wrap_mode(gtk.WRAP_WORD)
            self._builder.get_object("bugInfoDescriptionTextView").set_buffer(descBuffer)
            self._builder.get_object("bugInfoReportNoEntry").set_text(str(bug.bugnumber))
            self._builder.get_object("bugInfoReportDateEntry").set_text(str(bug.date))
            self._builder.get_object("bugInfoOpenInBrowserButton").set_uri(bug.url)
            
            for task in bug.infotable:
                self._addTask(task)
        else:
            self._builder.get_object("showReportersTaskListButton").set_sensitive(False)

    def _addTask(self, task):
        row = []
        if task.affects:
            row.append(task.affects)
        else:
            row.append("")
        if task.target:
            row.append(task.target[0].upper()+task.target[1:])
        else:
            row.append("")
        if task.targeted_to:
            row.append(task.targeted_to)
        else:
            row.append("")
        if task.status:
            row.append(task.status)
        else:
            row.append("")
        if task.importance:
            row.append(task.importance)
        else:
            row.append("")
        if task.assignee:
            row.append(task.assignee)
        else:
            row.append("")
        self._taskListModel.append(row)

    def _newBugInfoTaskListTreeviewColumn(self, text, id):
        col = gtk.TreeViewColumn(text, gtk.CellRendererText(), text=id)
        col.set_clickable(True)
        col.connect("clicked", self._onTaskColumnClicked, id)
        col.set_sort_indicator(True)
        self._bugInfoTaskListTreeview.append_column(col)
    
    def _onShowReportersTaskListButtonClicked(self, object):
        self.emit("show-reporters-task-list", self._bug.reporter)
