#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Nathan Samson
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

import gtk
import gobject

from leonov.gtk.objects.treeview import SimplifiedTreeView

class BugListTreeViewWidget(gtk.VBox):
    
    __gsignals__ = {
        "bug-selection-changed": (
                gobject.SIGNAL_RUN_FIRST,
                gobject.TYPE_NONE,
                (gobject.TYPE_INT,),
            ),
        "refresh-button-clicked" : (gobject.SIGNAL_RUN_FIRST,
                                                gobject.TYPE_NONE,
                                                (gobject.TYPE_INT, )),
                    "close-button-clicked"   : (gobject.SIGNAL_RUN_FIRST,
                                                gobject.TYPE_NONE,
                                                (gobject.TYPE_INT, ))}
    
    def __init__(self):
        
        gtk.VBox.__init__(self)
        
        self._hBox = gtk.HBox()
        self._hBox.show()
        self.pack_start(self._hBox, False, False)
        self._label = gtk.Label("Bugs of: ")
        self._label.show()
        self._hBox.pack_start(self._label, False, False)
        self._buttonClose = gtk.Button(None, gtk.STOCK_CLOSE)
        self._buttonClose.connect("clicked", self._onCloseButtonClicked)
        self._buttonClose.show()
        self._hBox.pack_end(self._buttonClose, False, False)
        self._buttonRefresh = gtk.Button(None, gtk.STOCK_REFRESH)
        self._buttonRefresh.connect("clicked", self._onRefreshButtonClicked)
        self._buttonRefresh.show()
        self._hBox.pack_end(self._buttonRefresh, False, False)
        
        self._store = gtk.TreeStore(str, str, str, object)
        self._treeview = SimplifiedTreeView(self._store)
        self._treeview.show()
        
        self._scrolledWindow = gtk.ScrolledWindow()
        self._scrolledWindow.set_policy(gtk.POLICY_AUTOMATIC,
                                        gtk.POLICY_AUTOMATIC)
        self._scrolledWindow.show()
        self.pack_end(self._scrolledWindow)
        self._scrolledWindow.add_with_viewport(self._treeview)
        
        self._treeview.add_columns(('Bug', 0), ('Summary', 2), ('Project', 1))
        self._treeview.set_enable_tree_lines(True)
        self._treeview.connect_selection_change(self._onBugSelectionChanged)
    
    def loadBugsForUser(self, bugList = None, user = None):
        self._initStore()
        
        for (itr, type) in [(self._iterAssigned, 0), (self._iterReported, 1),
                            (self._iterCommented, 2),
                            (self._iterSubscribed, 3)]:
            for bug in bugList[type]:
                self._store.append(itr, ["#"+str(bug.bugnumber), bug.sourcepackage,
                                         bug.summary, bug])
    
    def setLabel(self, lbl):
        self._label.set_text(lbl)
    
    def _initStore(self):
        # This should be changed so that _initStore does not remove root iters
        # (this collapses everything)
        self._store.clear()
        self._iterAssigned = self._store.append(None, ["Assigned", None, None, None])
        self._iterReported = self._store.append(None, ["Reported", None, None, None])
        self._iterCommented = self._store.append(None, ["Commented", None, None, None])
        self._iterSubscribed = self._store.append(None, ["Subscribed", None, None, None])

    def _onBugSelectionChanged(self, selection):
        (store, selIter) = selection.get_selected()
        if selIter:
            bug = self._store.get_value(selIter, 3)
            if bug != None:
                self.emit("bug-selection-changed", bug)
    
    def _onRefreshButtonClicked(self, object):
        self.emit("refresh-button-clicked", 0)
    
    def _onCloseButtonClicked(self, object):
        self.emit("close-button-clicked", 0)
