#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Nathan Samson
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

import gtk
import gobject

from leonov.gtk.buginfowidget import BugInfoWidget

class BugsNotebook(gtk.Notebook):
    
    __gsignals__ = {"show-reporters-task-list" : (gobject.SIGNAL_RUN_FIRST, 
                                                  gobject.TYPE_NONE,
                                                  (gobject.TYPE_STRING, ))}
    
    def __init__(self, builder):
        gtk.Notebook.__init__(self)
        self._builder = builder
        self._openTabs = {}
    
    def openCached(self, bugNumber):
        if self._openTabs.has_key(bugNumber):
            wdg = self._openTabs[bugNumber]
            n = self.page_num(wdg)
            self.set_current_page(n)
            return True
        else:
            return False
    
    def openBug(self, bug):
        widget = BugInfoWidget()
        lblWidget = self._generateLabelWidget(bug)
        n = self.append_page(widget, lblWidget)
        widget.showBug(bug)
        widget.show()
        self.set_current_page(n)
        self._openTabs[bug.bugnumber] = widget
        widget.connect("show-reporters-task-list", self._onReportersTaskList)
    
    def _onReportersTaskList(self, object, reporter):
        self.emit("show-reporters-task-list", reporter)
    
    def _generateLabelWidget(self, bug):
        hBox = gtk.HBox()
        
        lbl = gtk.Label("Bug report #"+str(bug.bugnumber))
        closeButton = gtk.Button()
        clsImage = gtk.Image()
        clsImage.set_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
        closeButton.set_image(clsImage)
        closeButton.connect("clicked", self._onCloseTab, bug.bugnumber)
        
        hBox.pack_start(lbl)
        hBox.pack_end(closeButton, False, False)
        hBox.show_all()
        return hBox
    
    def _onCloseTab(self, object, bugNumber):
        wdg = self._openTabs[bugNumber]
        n = self.page_num(wdg)
        self.remove_page(n)
        del self._openTabs[bugNumber]
