#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Stephan Hermann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

from PyQt4.QtGui import QTreeWidget, QTreeWidgetItem, QMessageBox
from PyQt4.QtCore import QObject, Qt, QVariant, SIGNAL, SLOT, QString, QStringList, QThread
from PyKDE4.kdecore import i18n
from PyKDE4.kdeui import KApplication
from leonov.backend import LPBugs, Global
from leonov.backend import config
from leonov.backend import LPDB_Users
from leonov.kde.models.threads import SQL_UserThread
from threads import GetBugListThread, GetBugThread, GetProjectListThread
import logging

class KLPBugs( QObject, LPBugs ):
    def __init__( self, parent=None, unstable=False ):
        QObject.__init__( self, parent )
        LPBugs.__init__( self, unstable )
    
    def throw_error( self, e ):
        qs=QString( str( e ) )
        self.emit( SIGNAL( 'LPBugs_Error(QString)' ), qs )
    
    def progress_hook( self, counter, block_size, size ):
        self.emit( SIGNAL( 'LPBugsRetrieve' ), counter, block_size, size )

klpbugs=KLPBugs(unstable=config.get_use_stable_launchpad())

class ProjectListModel(QObject):
    
    def __init__(self,parent=None):
        QObject.__init__(self,parent)
        self.initLPBugs()
        
    def initLPBugs(self):
        if klpbugs==None:
            self.lpBugs=KLPBugs()
        else:
            self.lpBugs=klpbugs
        self.connect(self.lpBugs,SIGNAL("LPBugs_Error(QString)"),self.LPBugErrors)
        self.connect(self.lpBugs,SIGNAL("LPBugsRetrieve"),self.slotProgress)
        
    def slotProgress(self,counter,bsize,total):
        print "Counter %d , bsize: %d , Total: %d" % (counter,bsize,total)
        
    def LPBugErrors( self, e ):
        QMessageBox.critical( None, 'Launchpad URL Error', e )    
    
    def fetchProjectList(self):
        logging.getLogger("leonov.kde.models.bugmodel").debug("fetchProjectList: Running")
        newThread=GetProjectListThread(self.lpBugs)
        self.connect(newThread,SIGNAL("receivedProjectList"),self.slotReceivedProjectList)
        newThread.start()
    
    def slotReceivedProjectList(self,projectlist):
        self.emit(SIGNAL("receivedProjectList"),projectlist)
        self.slotinsertDatabase(projectlist)
        
    def slotinsertDatabase(self,projectlist):
        # Pushing the list into the database
        if projectlist:
            sql_user_thread=SQL_UserThread(projectlist)
            sql_user_thread.start()

    
class BugListModel( QObject ):
    
    lpTypes={0:'assigned',
                    1:'reported',
                    2:'commented',
                    3:'subscribed'}
    def __init__( self, parent=None ):
        QObject.__init__( self, parent )
        self.initLPBugs()
        self.setThreadSlots()
    def initLPBugs( self ):
        if klpbugs == None:
            self.lpBugs=KLPBugs()
        else:
            self.lpBugs=klpbugs
        self.connect( self.lpBugs, SIGNAL( "LPBugs_Error(QString)" ), self.LPBugErrors )
    
    def setThreadSlots( self, started=None, finished=None, terminated=None ):
        self._started=started
        self._finished=finished
        self._terminated=terminated
        
    def LPBugErrors( self, e ):
        QMessageBox.critical( None, 'Launchpad URL Error', e )
        
    def fetchBugList( self, lpType, personal=True,  username="" ):
        logging.getLogger("leonov.kde.models.bugmodel.BugListModel").debug("fetchBugList")
        newThread=GetBugListThread( lpType, self.lpBugs, personal, username )
        self.connect( newThread, SIGNAL( "receiveBugList" ), self.receivedBugList )
        if self._started!=None:
            self.connect( newThread, SIGNAL( "started()" ), self._started )
        if self._finished!=None:
            self.connect( newThread, SIGNAL( "finished()" ), self._finished )
        if self._terminated!=None:
            self.connect( newThread, SIGNAL( "terminated()" ), self._terminated )
        newThread.start()

    def receivedBugList( self, buglist, lpType ):
        self.emit( SIGNAL( "receiveBugList" ), buglist, lpType )
         
class BugModel( QObject ):
    
    def __init__( self, parent=None ):
        QObject.__init__( self, parent )
        if klpbugs == None:
            self.lpBugs=KLPBugs()
        else:
            self.lpBugs=klpbugs

    def getBugReport( self, bugno ):
        self.bugno=bugno
        newThread=GetBugThread( self.bugno, self.lpBugs )
        self.connect( newThread, SIGNAL( "BugReportFetched" ), self.gotBugReport )
        newThread.run()

    def gotBugReport( self, bugReport ):
        self.emit( SIGNAL( "receiveBugReport" ), bugReport )
        
    def setContentType(self,content_type):
        self.lpBugs.setContentType(content_type)
        
        
    def prepareTaskEntry(self,buginfo):
        entry=QStringList()
        if buginfo.affects:
            entry.append(buginfo.affects)
        else:
            entry.append("")
        if buginfo.target:
            entry.append(buginfo.target[0].upper()+buginfo.target[1:])
        else:
            entry.append("")
        if buginfo.targeted_to:
            entry.append(buginfo.targeted_to)
        else:
            entry.append("")
        if buginfo.status:
            entry.append(buginfo.status)
        else:
            entry.append("")
        if buginfo.importance:
            entry.append(buginfo.importance)
        else:
            entry.append("")
        if buginfo.assignee:
            entry.append(buginfo.assignee)
        else:
            entry.append("")
        return entry
    
    def prepareCommentEntry(self,counter,comment):
        entry=QStringList()
        if counter:
            entry.append(str(counter))
        else:
            entry.append("")
        if comment.user:
            entry.append(comment.user.realname)
        else:
            entry.append("")
        if comment.subject:
            entry.append(comment.subject)
        else:
            entry.append("")
        if comment.date:
            entry.append(str(comment.date))
        else:
            entry.append("")
        return entry
            
        
        
