# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Stephan Hermann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################
from PyKDE4.kdeui import KApplication, KDialog, KLineEdit
from PyKDE4.kdecore import KUrl 
from PyKDE4.kio import KFileDialog
from PyKDE4.kdecore import i18n
from PyQt4.QtGui import QWidget, QCheckBox, QToolButton
from PyQt4.QtCore import SIGNAL, SLOT, QObject

from UI import Ui_viewLaunchpadSettings

from leonov.backend import config, Global
import logging
import os

class _ViewLaunchpadSettings(QWidget):
    
    def __init__(self, parent=None):        
        QWidget.__init__(self, parent)
        self.ui=Ui_viewLaunchpadSettings()
        self.ui.setupUi(self)
        self.setupSlots()

    def setupSlots(self):
        self.connect(self.ui.btnAttachmentDownloadPath,SIGNAL("clicked()"),self.slotBtnAttachmentDownloadPath)
        
    def getAutoLogin(self):
        return self.ui.checkAutoLogin.isChecked()

    def getAuthPerDay(self):
        return self.ui.checkAuthPerDay.isChecked()
    
    def getAttachmentDownloadPath(self):
        return self.ui.editAttachmentDownloadPath.text()
    def getBetaTestUser(self):
        return self.ui.checkLPBetaTestUser.isChecked()

    def setAutoLogin(self, boolean):
        self.ui.checkAutoLogin.setChecked(boolean)

    def setAuthPerDay(self, boolean):
        print boolean
        self.ui.checkAuthPerDay.setChecked(boolean)
        
    def setAttachmentDownloadPath(self,download_path):
        self.ui.editAttachmentDownloadPath.setText(download_path)
    def slotBtnAttachmentDownloadPath(self):
        saveUrl=KFileDialog.getExistingDirectoryUrl(KUrl(""),self,i18n("Attachment Path"))
        if saveUrl!="":
            self.setAttachmentDownloadPath(saveUrl.path())
    def setBetaTestUser(self,beta=False):
        self.ui.checkLPBetaTestUser.setChecked(beta)
        
class ViewLaunchpadSettings(QObject):

    def __init__(self,  parent=0):
        QObject.__init__(self, parent)
        self.dlg=KDialog(parent)
        self.dlg.setCaption(i18n("Leonov Preferences"))
        self.dlg.setButtons(KDialog.ButtonCode(KDialog.Ok | KDialog.Cancel))
        self.connect(self.dlg,SIGNAL("okClicked()"), self.saveSettings)
        self.lpSettings=_ViewLaunchpadSettings(self.dlg)
        self.dlg.setMainWidget(self.lpSettings)
        authperday = config.get_auth_per_day()
        stayauth = config.get_stay_authenticated()
        betatestuser = config.get_use_stable_launchpad()
        self.lpSettings.setAuthPerDay(authperday)
        self.lpSettings.setAutoLogin(stayauth)
        self.lpSettings.setBetaTestUser(betatestuser)
        if config.get_attachment_download_path()=="":
            self.lpSettings.setAttachmentDownloadPath(os.getcwd())
        else:
            self.lpSettings.setAttachmentDownloadPath(config.get_attachment_download_path())

    def exec_(self):
        return self.dlg.exec_()
    
    def saveSettings(self):
        config.set_stay_authenticated(self.lpSettings.getAutoLogin())
        config.set_auth_per_day(self.lpSettings.getAuthPerDay())
        config.set_use_stable_launchpad(self.lpSettings.getBetaTestUser())
        if not self.lpSettings.getAttachmentDownloadPath():
            config.set_attachment_download_path(str(self.lpSettings.getAttachmentDownloadPath()))


