#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Stephan Hermann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

from PyKDE4.kdeui import KApplication, KTabWidget, KMenu
from PyKDE4.kio import KFileDialog
from PyKDE4.kdecore import i18n, KUrl
from PyQt4.QtGui import QWidget, QVBoxLayout, QTreeWidget, QTreeWidgetItem, QPushButton, QCursor, QMdiSubWindow, QDesktopServices
from PyQt4.QtCore import SIGNAL, SLOT, QStringList, Qt, QString

from UI import Ui_viewBugInfo
from UI import Ui_tabBugTasks
from UI import Ui_tabBugComments
from UI import Ui_tabBugActivity
from UI import Ui_tabBugAdditional
from UI import Ui_tabPageBugInfoBranches
from UIHelpers import AttachmentContextMenu, tabPage

from leonov.backend import lpbackend, config, Global
from leonov.kde.models import klpbugs, BugModel

import logging,os

class tabPageBugInfoBranchesWidget(tabPage):
    def __init__(self,parent=None,bugmodel=None):
        tabPage.__init__(self,parent,bugmodel,Ui_tabPageBugInfoBranches)
        self.setupView()
        self.setupSlots()
        self.branches={}
        
    def setupView(self):
        self.ui.setupUi(self)
        self.setupHeaderLines()
    
    def setupHeaderLines(self):
        headerItems=QStringList()
        headerItems << i18n("Branch") << i18n("Status")
        self.ui.treeBranches.setHeaderLabels(headerItems)
        
    def setupSlots(self):
        self.connect(self.bugModel,SIGNAL("receiveBugReport"),self.receiveBugReport)
        self.connect(self.ui.treeBranches,SIGNAL("currentItemChanged(QTreeWidgetItem *,QTreeWidgetItem *)"),self.slotItemChanged)
        self.connect(self.ui.btnVisitHomepage,SIGNAL("clicked()"),self.slotBtnVisitHomepage)
        
    def receiveBugReport(self,bugreport):
        self.bugReport=bugreport
        if self.bugReport.branches:
            self.emit(SIGNAL("tabPageEnable"),self,True)
            self.prepareBranchEntries()
        else:
            self.emit(SIGNAL("tabPageEnable"),self,False)
        
    def prepareBranchEntries(self):
        if self.bugReport.branches:
            counter=0
            for i in self.bugReport.branches:
                self.branches[counter]=i
                entries=QStringList()
                entries << i.title << i.status
                QTreeWidgetItem(self.ui.treeBranches,entries)
    def slotItemChanged(self,current_item,previous_item):
        if current_item:
            self.ui.btnVisitHomepage.setEnabled(True)
        else:
            self.ui.btnVisitHomepage.setEnabled(False)
    def slotBtnVisitHomepage(self):
        current_item=self.ui.treeBranches.currentItem()
        if current_item:
            current_index=self.ui.treeBranches.indexOfTopLevelItem(current_item)
            if self.branches[current_index]:
                QDesktopServices.openUrl(KUrl(self.branches[current_index].url))
                

class tabPageBugInfoAdditionalWidget(tabPage):
    def __init__(self,parent=None,bugmodel=None):
        tabPage.__init__(self,parent,bugmodel,Ui_tabBugAdditional)
        # self.bugModel=bugmodel
        # self.ui=Ui_tabBugAdditional()
        self.rootItems={}
        self.setupView()
        self.setupSlots()
        
    def setupView(self):
        self.ui.setupUi(self)
        self.setupHeaderLines()
        self.setupRootItems()
        
    def setupHeaderLines(self):
        headerItems=QStringList()
        headerItems << i18n("Type") << i18n("Subscribers Name")
        self.ui.treeSubscribers.setHeaderLabels(headerItems)
        headerItems=QStringList()
        headerItems << i18n("Task No.")
        self.ui.treeDuplicates.setHeaderLabels(headerItems)
    def setupRootItems(self):
        self.rootItems["directly"]=QTreeWidgetItem(self.ui.treeSubscribers,QStringList() << i18n("Subscribed themselves"))
        self.rootItems["duplicates"]=QTreeWidgetItem(self.ui.treeSubscribers,QStringList() << i18n("via Duplicates"))
        self.rootItems["notified"]=QTreeWidgetItem(self.ui.treeSubscribers,QStringList() << i18n("Also notified"))
        for i in self.rootItems.keys():
            self.ui.treeSubscribers.addTopLevelItem( self.rootItems[i] )
    
    def prepareSubscribers(self):
        direct_list=self.bugReport.get_subscriptions_category("directly")
        via_duplicates=self.bugReport.get_subscriptions_category("duplicates")
        also_notified=self.bugReport.get_subscriptions_category("notified")
        for i in self.rootItems.keys():
            listing=self.bugReport.get_subscriptions_category(i)
            for user in listing:
                entry=QStringList()
                entry.append("")
                if user.realname:
                    entry.append(user.realname+" ("+str(user)+") ")
                else:
                    entry.append(str(user))
                QTreeWidgetItem(self.rootItems[i],entry)
        
    def prepareDuplicates(self):
        if self.bugReport.duplicates:
            for dupe in self.bugReport.duplicates:
                QTreeWidgetItem(self.ui.treeDuplicates,QStringList() << str(dupe))
        
    def setupSlots(self):
        self.connect(self.bugModel,SIGNAL("receiveBugReport"),self.receiveBugReport)    
        
    def receiveBugReport(self,bugreport):
        self.bugReport=bugreport
        self.prepareSubscribers()
        self.prepareDuplicates()
    

class tabPageBugInfoActivityWidget(tabPage):
    def __init__(self,parent=None,bugmodel=None):
        tabPage.__init__(self,parent,bugmodel,Ui_tabBugActivity)
        self.setupView()
        self.setupSlots()
        
    def setupView(self):
        self.ui.setupUi(self)
        self.initActivityList()
        
    def initActivityList(self):
        headerItems=QStringList()
        headerItems << i18n("Date") << i18n("Who") << i18n("What changed") << i18n("Old Value") << i18n("New value") << i18n("Message")
        self.ui.treeActivity.setHeaderLabels(headerItems)


    def setupSlots(self):
        self.connect(self.bugModel,SIGNAL("receiveBugReport"),self.receiveBugReport)
        
    def receiveBugReport(self,bugreport):
        self.bugReport=bugreport
        self.prepareActivityEntries()
    def prepareActivityEntries(self):
        if self.bugReport.activity:
            for entry in self.bugReport.activity:
                entryList=QStringList()
                if entry.date:
                    entryList.append(str(entry.date))
                else:
                    entryList.append("")
                if entry.user:
                    entryList.append(entry.user.realname)
                else:
                    entryList.append("")
                if entry.what:
                    entryList.append(entry.what)
                else:
                    entryList.append("")
                if entry.old_value:
                    entryList.append(entry.old_value)
                else:
                    entryList.append("")
                if entry.new_value:
                    entryList.append(entry.new_value)
                else:
                    entryList.append("")
                if entry.message:
                    entryList.append(entry.message)
                else:
                    entryList.append("")
                QTreeWidgetItem(self.ui.treeActivity,entryList)     

class tabPageBugInfoTasksWidget(tabPage):
    
    def __init__(self,parent=None, bugmodel=None):
        tabPage.__init__(self,parent,bugmodel,Ui_tabBugTasks)
        self.setupView()
        self.setupSlots()
        self.connect(self.bugModel,SIGNAL("receiveBugReport"),self.receiveBugReport)

    def setupView(self):
        self.ui.setupUi(self)
        self.initTaskList()

    def initTaskList(self):
        # Setup header columns
        headerItems=QStringList()
        headerItems << i18n("Task") << i18n("Project/Distro") << i18n("Release") << i18n("Status") << i18n("Importance") << i18n("Assignee")
        self.ui.treeTaskList.setHeaderLabels(headerItems)

    def setupSlots(self):
        self.connect(self.ui.btnShowBugsOf,SIGNAL("clicked()"),self.visitTasksOf)
        self.connect(self.ui.btnShowBrowser,SIGNAL("clicked()"),self.slotBtnShowBrowser)
        
    def slotBtnShowBrowser(self):
        logging.getLogger("leonov.kde.views.mainbuginfo").debug("slotBtnShowBrowser: clicked")
        QDesktopServices.openUrl(KUrl(self.bugReport.url))
    def visitTasksOf(self):
        self.emit(SIGNAL("BugInfoVisitTasksOf"),self.bugReport.reporter)


    def addTasks(self, infotable):        
        for i in infotable:
            self.bugModel.addTask(i)
        pass
    def setBugNumber(self, bugNumber):
        self.ui.editBugNumber.setText(str(bugNumber))
    def setBugSummary(self, summary):
        self.ui.editSummary.setText(summary)
        self.ui.editSummary.setCursorPosition(0)
    def setBugReporter(self,reporter):
        self.ui.editBugReporter.setText(reporter.realname+" ("+reporter+")")
        self.ui.editBugReporter.setCursorPosition(0)
    def setBugReportDate(self,lptimedate):
        self.ui.editFirstReported.setText(str(lptimedate))
        self.ui.editFirstReported.setCursorPosition(0)
    def setBugDescription(self,description):
        self.ui.editDescription.setText(description)
    def receiveBugReport(self, bugreport):
        logging.getLogger("leonov.kde.views.tabPageBugInfoTaskWidet").debug("received Bugreport")
        self.ui.treeTaskList.clear()
        self.ui.btnShowBugsOf.setEnabled(True)
        self.bugReport=bugreport
        self.setBugNumber(bugreport.bugnumber)
        self.setBugSummary(bugreport.title)
        self.addTasks(bugreport.infotable)
        self.setBugReporter(bugreport.reporter)
        self.setBugReportDate(bugreport.date_reported)
        self.setBugDescription(bugreport.description)
        
    def addTasks(self,infoTable):
        self.ui.treeTaskList.hide()
        for info in infoTable:
            QTreeWidgetItem(self.ui.treeTaskList,self.bugModel.prepareTaskEntry(info))
        self.ui.treeTaskList.show()

class tabPageBugInfoCommentsWidget(tabPage):
    def __init__(self,parent=None, bugmodel=None):
        tabPage.__init__(self,parent,bugmodel,Ui_tabBugComments)
        self.setupView()
        self.showAllAttachments=True
        self.setupCommentList()
        self.setupSlots()
        self.connect(self.bugModel,SIGNAL("receiveBugReport"),self.receiveBugReport)
        self.comments=[]
        
    def setupView(self):
        self.ui.setupUi(self)
        self.ui.checkAllAttachments.setChecked(True)
    def setupCommentList(self):
        headerItems=QStringList()
        headerItems << i18n("No.") << i18n("Commentor") << i18n("Subject") << i18n("Date")
        self.ui.treeCommentList.setHeaderLabels(headerItems)
        headerItems=QStringList()
        headerItems << i18n("Filename") << i18n("Content Type")
        self.ui.treeAttachments.setHeaderLabels(headerItems)
        
    def setupSlots(self):
        self.connect(self.ui.treeCommentList,SIGNAL("currentItemChanged(QTreeWidgetItem *,QTreeWidgetItem *)"),self.itemChanged)
        self.connect(self.ui.checkAllAttachments,SIGNAL("stateChanged (int)"),self.slotAllAttachments)
        self.connect(self.ui.treeAttachments,SIGNAL("customContextMenuRequested( const QPoint &)"),self.slotAttachmentMenu)
    
    def slotAttachmentMenu(self, point_xy):
        if self.ui.treeAttachments.topLevelItemCount()>0:
            #
            # do we have multiple selections?
            # check for that first 
            #
            if len(self.ui.treeAttachments.selectedItems())>0:
                popup=AttachmentContextMenu(self,"Save Attachments Do Something",str(self.ui.treeAttachments.currentItem().text(0)),len(self.ui.treeAttachments.selectedItems()))
                act=popup.getMenu().exec_(QCursor.pos())
                if act:
                    for attachment in self.bugreport.attachments:
                        if attachment.lp_filename==self.ui.treeAttachments.currentItem().text(0):
                            if act.data().toString()=="SaveAttachment":
                                # save attachment to default location:
                                if config.get_attachment_download_path()=="":
                                    self.bugModel.setContentType(attachment.contenttype)
                                    attachment.download("./"+attachment.lp_filename)
                                else:
                                    self.bugModel.setContentType(attachment.contenttype)
                                    attachment.download(config.get_attachment_download_path()+"/"+attachment.lp_filename)
                            elif act.data().toString()=="SaveAttachmentTo":
                                saveUrl=KUrl(os.getcwd())
                                saveUrl.addPath(attachment.lp_filename)
                                saveFile=KFileDialog.getSaveFileName(saveUrl,"*",self,i18n("Save Attachment to..."))
                                if saveFile:
                                    self.bugModel.setContentType(attachment.contenttype)
                                    attachment.download(str(saveFile))
                            elif act.data().toString()=="SaveAll" or act.data().toString()=="SaveAllTo":
                                if act.data().toString()=="SaveAllTo":
                                    saveUrl=KUrl("kfiledialog:///leonov-kde")
                                    saveDir=str(KFileDialog.getExistingDirectory(saveUrl,self,i18n("Save all Attachments to...")))
                                else:
                                    saveDir=config.get_attachment_download_path()
                                for entry in self.ui.treeAttachments.selectedItems():                                    
                                    for attachment in self.bugreport.attachments:
                                        if attachment.lp_filename==str(entry.text(0)):
                                            self.bugModel.setContentType(attachment.contenttype)
                                            attachment.download(saveDir+"/"+attachment.lp_filename)                                                
                
    def slotAllAttachments(self,checked):
        if checked==Qt.Checked:
            self.showAllAttachments=True
        else:
            self.showAllAttachments=False
        logging.getLogger("slotAllAttachments").debug("self.showAllAttachments %d",int(self.showAllAttachments))
        self.updateAttachmentView()
        
    def updateAttachmentView(self,attachments=None):
        if self.showAllAttachments:
            self.ui.treeAttachments.clear()
            if self.bugreport.attachments:
                self.prepareAttachments(self.bugreport.attachments)
        else:
            logging.getLogger("updateAttachmentView").debug("self.showAllAttachments %d",int(self.showAllAttachments))
            self.ui.treeAttachments.hide()
            self.ui.treeAttachments.clear()
            if attachments:
                self.prepareAttachments(attachments)
            self.ui.treeAttachments.show()
        pass
    def itemChanged(self,item1,item2):
        if item1:
            self.ui.editCommentText.setText(self.bugreport.comments[int(item1.text(0))-1].text)
            if not self.showAllAttachments:
                if self.bugreport.comments[int(item1.text(0))-1].attachments:
                    self.updateAttachmentView(self.bugreport.comments[int(item1.text(0))-1].attachments)
                else:
                    self.ui.treeAttachments.clear()
        pass
    def prepareAttachments(self,attachments):
        self.ui.treeAttachments.hide()
        for attachment in attachments:
            QTreeWidgetItem(self.ui.treeAttachments,QStringList() << attachment.lp_filename << attachment.contenttype)
        self.ui.treeAttachments.show()
    def receiveBugReport(self, bugreport):
        logging.getLogger("leonov.kde.views.tabPageBugInfoGeneralWidget").debug("received Bugreport")
        self.bugreport=bugreport
        self.addComments(bugreport.comments)
        self.updateAttachmentView()
    def addComments(self,commentlist):
        self.ui.treeCommentList.hide()
        self.ui.treeCommentList.clear()
        self.ui.editCommentText.clear()
        counter=0
        for comment in commentlist:
            counter=counter+1
            self.prepareComment(counter,comment)
        self.ui.treeCommentList.show()
        
    def prepareComment(self,counter,comment):
        temp=QTreeWidgetItem(self.ui.treeCommentList,self.bugModel.prepareCommentEntry(counter,comment))

class mainBugInfoWidget(QWidget):
    
    def __init__(self,parent=None,title=""):
        QWidget.__init__(self,parent)
        self.ui=Ui_viewBugInfo()
        self.bugModel=BugModel(self)
        self.connect(self.bugModel,SIGNAL("receiveBugReport"),self.receiveBugReport)
        self.setupView()
        self.setWindowTitle(title)
        self.setupSlots()

    def setupView(self):
        self.ui.setupUi(self)
        self.bugComments=tabPageBugInfoCommentsWidget(bugmodel=self.bugModel)
        self.bugTasks=tabPageBugInfoTasksWidget(bugmodel=self.bugModel)
        self.bugActivity=tabPageBugInfoActivityWidget(bugmodel=self.bugModel)
        self.bugAdditionalInfo=tabPageBugInfoAdditionalWidget(bugmodel=self.bugModel)
        self.bugBranches=tabPageBugInfoBranchesWidget(bugmodel=self.bugModel)
        self.addBugPages()

    def addBugPages(self):
        self.ui.tabBugInfo.addTab(self.bugTasks,i18n("General"))
        self.ui.tabBugInfo.addTab(self.bugComments,i18n("Comments"))
        self.ui.tabBugInfo.addTab(self.bugBranches,i18n("Branches"))
        self.ui.tabBugInfo.addTab(self.bugActivity,i18n("Activity"))
        self.ui.tabBugInfo.addTab(self.bugAdditionalInfo,i18n("Additional Informations"))
        
    def setupSlots(self):
        self.connect(self.bugTasks,SIGNAL("BugInfoVisitTasksOf"),self.buginfovisit)
        self.connect(self.bugBranches,SIGNAL("tabPageEnable"),self.slotTabPageEnable)
    def slotTabPageEnable(self,widget,enabled):
        if widget:
            self.ui.tabBugInfo.setTabEnabled(self.ui.tabBugInfo.indexOf(widget),enabled)
    def buginfovisit(self,bugreporter):
        self.emit(SIGNAL("BugInfoVisitTasksOf"),bugreporter)
        
    def bugRequested(self, bugno):
        self.bugModel.getBugReport(bugno)
    def receiveBugReport(self,bugreport):
        #self.show()
        pass
