#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Stephan Hermann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

from PyKDE4.kdeui import KApplication,  KButtonGroup
from PyKDE4.kdecore import i18n
from PyQt4.QtGui import QLabel, QTabWidget, QWidget,  QGridLayout,  QHBoxLayout,  QRadioButton, QColumnView,  QMessageBox,  QTreeWidget,  QTreeWidgetItem, QDockWidget
from PyQt4.QtCore import QSize, QRect,  SIGNAL,  SLOT,  QStringList, Qt
from UI import Ui_tabPBS
import logging
from leonov.kde.models import BugListModel, klpbugs

class tabPersonalBugs(QWidget):
    
    lpTypes={0:'assigned', 
             1:'reported', 
             2:'commented', 
             3:'subscribed'}
    
    def __init__( self, parent=0, personal=True, username="" ):
        QWidget.__init__( self, parent )
        self.ui=Ui_tabPBS()
        self.ui.setupUi( self )
        self.rootItems={}
        self.setupModel()
        self.setupSlots()
        self.personal=personal
        self.username=username
        self.buglist_counter=1
        
    def setupSlots(self):
        self.connect(self.ui.treePersonalBugs,SIGNAL("itemDoubleClicked(QTreeWidgetItem *,int)"),self.doubleClick)
        self.connect(self.bugModel,SIGNAL("receiveBugList"),self.receivedBugList)
    
    def setupModel(self):
        self.bugModel=BugListModel(self)
        self.setupTreeList()
        
    def setupTreeList(self):
        self.ui.treePersonalBugs.setColumnCount(3)
        headerItems=QStringList() 
        headerItems << i18n( "Type" ) << i18n( "Report No." ) << i18n( "Summary" )
        self.ui.treePersonalBugs.setHeaderLabels( headerItems )
        self.rootItems["assigned"]=QTreeWidgetItem( self.ui.treePersonalBugs, QStringList() << i18n( "Assigned" ) << "" << "" )
        self.rootItems["reported"]=QTreeWidgetItem( self.ui.treePersonalBugs, QStringList() << i18n( "Reported" ) << "" << "" )
        self.rootItems["commented"]=QTreeWidgetItem( self.ui.treePersonalBugs, QStringList() << i18n( "Commented" ) << "" << "" )
        self.rootItems["subscribed"]=QTreeWidgetItem( self.ui.treePersonalBugs, QStringList() << i18n( "Subscribed" ) << "" << "" )        
        for i in self.rootItems.keys():
            self.ui.treePersonalBugs.addTopLevelItem( self.rootItems[i] )
        self.ui.treePersonalBugs.sortItems(1,Qt.DescendingOrder)
            
    def fetchBugs(self):
        # Clear now the children of the root items
        self.clearChildren()      
        for i in self.lpTypes.keys():
            self.buglist_counter+=1
            self.emit(SIGNAL("fetchBugsInProgress"),1,4,-1)
            self.bugModel.fetchBugList(i,self.personal,self.username)
            
            
    def clearChildren(self):
        self.ui.treePersonalBugs.hide()
        for rootItem in self.lpTypes.keys():
            self.rootItems[self.lpTypes[rootItem]].takeChildren()
        self.ui.treePersonalBugs.show()

    def receivedBugList(self,buglist, lpType):
        logging.getLogger("leonov.kde.views.receiveBugList").debug("receivedBugList: BugList %s received",self.lpTypes[lpType])
        self.addBugs(buglist,lpType)
        self.buglist_counter-=1
        logging.getLogger("leonov.kde.views.receiveBugList").debug("receivedBugList: self.buglist_counter: %d",self.buglist_counter)
        if self.buglist_counter==1:
            self.emit(SIGNAL("fetchBugsInProgress"),0,4,1)
        else:
            self.emit(SIGNAL("fetchBugsInProgress"),2,4,self.buglist_counter)
        
    def addBugs(self,buglist,lpType):
        self.ui.treePersonalBugs.hide()
        for bugentry in buglist:
            QTreeWidgetItem(self.rootItems[self.lpTypes[lpType]],QStringList() << "" << str(bugentry.bugnumber) << bugentry.summary)
        self.ui.treePersonalBugs.show()
        
    def doubleClick(self, treeWidgetItem, column):
        logging.getLogger("leonov.kde.views.mainwidgets.tabPersonalBugs").debug("doubleClick")
        if treeWidgetItem.text(0)=="":
            self.emit(SIGNAL("LPBugRequest(int)"),int(treeWidgetItem.text(1)))
            
class MyBugListDocker(QDockWidget):

    def __init__(self,parent=None, title=""):
        QDockWidget.__init__(self,title,parent)
        
    def fetchBugs(self,personal=True,username=""):
        if personal:
            self.widget().fetchBugs()
        else:
            self.widget().fetchBugs()
    def setupSlots(self):
        self.connect(self.widget(),SIGNAL("LPBugRequest(int)"), self.slotBugRequest)
        self.connect(self.widget(),SIGNAL("fetchBugsInProgress"),self.slotFetchBugsInProgress)
    def slotBugRequest(self, bugno):
        self.emit(SIGNAL("LPBugRequest(int)"),bugno)
    def slotFetchBugsInProgress(self,boolean,max_counter,counter):
        logging.getLogger("leonov.kde.views.mainwidgets.slotFetchBugsInProgress").debug("fetchBugsInProgress")
        self.emit(SIGNAL("fetchBugsInProgress"),boolean,max_counter,counter)
