#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Stephan Hermann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################

import os.path, sys
from PyKDE4.kdeui import KApplication, KXmlGuiWindow,  KAction,  KProgressDialog,KStandardAction,  KActionCollection,  KStatusBar,  KTextEdit
from PyKDE4.kdecore import i18n
from PyQt4.QtGui import QLabel, QTabWidget, QStatusBar, QProgressBar, QDockWidget, QDialog, QMdiArea, QMdiSubWindow
from PyQt4.QtCore import QSize,  SIGNAL,  SLOT,  Qt
from PyQt4.uic import loadUi
from mainwidgets import tabPersonalBugs, MyBugListDocker
from mainlogin import ViewLoginDlg
from mainbuginfo import mainBugInfoWidget
from maintabview import mainTabViewWidget
from dlggettaskreport import DlgGetTaskReport
from dlglaunchpadsettings import ViewLaunchpadSettings
from dlggetlaunchpadusername import DlgGetLaunchpadUsername
from winprojectnewreport import windowProjectNewReport
from leonov.backend import config
from leonov.kde.models import klpbugs
from leonov.kde.models.threads import LoginThread
from leonov.kde.models import ProjectListModel
from leonov.backend import Global
import re
import logging



class MainWindow (KXmlGuiWindow):
    def __init__(self):
        KXmlGuiWindow.__init__(self)
        # Login method
        # do this everytime we start
        self.kApp=KApplication.kApplication()            
        self.myDockActions=[]
        self.setupView()
        self.dockWidgets={}
        self.setupActions()
        self.setupDockWidgets()
        self.setupSlots()
        # self.setCentralWidget(self.mainBugInfo)
        # add mdi area as central widget
        self.setCentralWidget(self.mainTaskView)
        self.setupGUI(QSize (600, 400), KXmlGuiWindow.Default, os.path.join (sys.path [0],"leonov/kde/leonovui.rc"))
        # self.setupGUI(QSize (600, 400), KXmlGuiWindow.Default, "leonovui.rc")
        self.addActions()
        self.bugWindows={}
        self.bugWidgets={}
        self.findTaskNo=re.compile(".*#(\d+)")
        if Global.cookie_file and not os.path.exists(Global.cookie_file):
            self.launchpad_login()
        else:
            klpbugs.authenticateLPUserViaCookie()
            self.updateStatusBar(i18n("Logged in as %1",config.get_username()), 0)
        self.initData()
        
    def initData(self):
        self.projectListModel=ProjectListModel(self)
        self.projectListModel.fetchProjectList()
        
    def setupActions(self):
        self.fetch_bugs=KAction(self)
        self.fetch_bugs.setText(i18n("Fetch Bugs"))
        self.actionCollection().addAction("fetch_bugs", self.fetch_bugs)
        self.connect(self.fetch_bugs, SIGNAL("activated()"), self.fetchBugsClicked)
        KStandardAction.quit(self.kApp.quit, self.actionCollection())
        KStandardAction.preferences(self.leonov_preferences,self.actionCollection())
        self.viewmode_set_tab_position=KAction(self)
        self.viewmode_set_tab_position.setText(i18n("Tabs at the Top"))
        self.viewmode_set_tab_position.setCheckable(True)
        self.viewmode_set_tab_position.setChecked(True)
        self.actionCollection().addAction("viewmode_set_tab_position",self.viewmode_set_tab_position)
        self.connect(self.viewmode_set_tab_position,SIGNAL("toggled(bool)"),self.slotViewmodeSetTabPos)
        self.launchpad_get_task_report=KAction(self)
        self.launchpad_get_task_report.setText(i18n("Get Task Report"))
        self.actionCollection().addAction("get_task_report",self.launchpad_get_task_report)
        self.connect(self.launchpad_get_task_report,SIGNAL("activated()"),self.slotLaunchpadGetTaskReport)
        
        self.launchpad_get_task_list_of=KAction(self)
        self.launchpad_get_task_list_of.setText(i18n("Get Tasklist of ..."))
        self.actionCollection().addAction("get_task_list_of",self.launchpad_get_task_list_of)
        self.connect(self.launchpad_get_task_list_of,SIGNAL("activated()"),self.slotLaunchpadGetTasklistOf)
        
        self.launchpad_new_project_report=KAction(self)
        self.launchpad_new_project_report.setText(i18n("File new Project Report"))
        self.actionCollection().addAction("new_project_report",self.launchpad_new_project_report)
        self.connect(self.launchpad_new_project_report,SIGNAL("activated()"),self.slotLaunchpadNewProjectReport)
                     
    def slotLaunchpadGetTaskReport(self):
        # dialog box for task report bug no entry
        dlg=DlgGetTaskReport(self)
        if dlg.exec_()==QDialog.Accepted:
            self.addBugInfoWindow(dlg.getTaskReportNumber())
        pass
    
    def slotLaunchpadGetTasklistOf(self):
        dlg=DlgGetLaunchpadUsername(self)
        if dlg.exec_()==QDialog.Accepted:
            self.slotVisitTaskOf(dlg.getLPUsername())
            
    def slotViewmodeSetTabPos(self, tab_pos):
        if tab_pos:
            self.mainTaskView.setTabPosition(QTabWidget.North)
        else:
            self.mainTaskView.setTabPosition(QTabWidget.South)
        
    def fetchBugsClicked(self):
        if self.dockWidgets.has_key("personal"):
            self.dockWidgets["personal"].fetchBugs()
        else:
            self.setupDockWidgets()
    
    def leonov_preferences(self):
        prefDlg=ViewLaunchpadSettings(self)
        if prefDlg.exec_()==QDialog.Accepted:
            config.save()

    def launchpad_login(self):
        loginDlg=ViewLoginDlg(self)
        if loginDlg.exec_()==QDialog.Accepted:
            config.load()
            self.progressDialog=KProgressDialog(self,i18n("Logging into Launchpad..."))
            self.progressDialog.progressBar().setRange(0,2)
            self.progressDialog.progressBar().setValue(0)
            self.progressDialog.setAllowCancel(False)
            self.progressDialog.setModal(True)
            self.progressDialog.show()
            self.loginThread=LoginThread(klpbugs,config.get_username(),loginDlg.getPassword())
            self.connect(self.loginThread,SIGNAL("started()"),self.slotLoginThreadStarted)
            self.connect(self.loginThread,SIGNAL("finished()"),self.slotLoginThreadFinished)
            self.loginThread.start()

        else:
            logging.getLogger("leonov.kde.mainwindow").warning("Login Dialog Canceled")
            sys.exit(0)

    def slotLoginThreadStarted(self):
        print "slotLoginThreadStarted"
        self.progressDialog.progressBar().setValue(1)
        self.progressDialog.setWindowTitle(i18n("Logging into Launchpad..."))
        
    def slotLoginThreadFinished(self):
        self.progressDialog.progressBar().setValue(2)
        self.progressDialog.setWindowTitle(i18n("Logged in as %1",config.get_username()))
        self.statusBar().changeItem(i18n("Logged in as %1",config.get_username()),0)
        self.progressDialog.hide()
        del self.progressDialog
    def setupView(self):
        self.createStandardStatusBarAction()
        statusBar=self.statusBar()
        self.progressBar=QProgressBar(self)
        self.progressBar.setMaximum(1)
        self.progressBar.setMinimum(1)
        statusBar.insertItem("", 0)
        statusBar.insertItem("Status", 1)
        statusBar.addPermanentWidget(self.progressBar)
        self.mainTaskView=mainTabViewWidget(self)

        
    def setupDockWidgets(self):
        self.dockWidgets["personal"]=self.createDockBugList(i18n("Personal Tasks"),"personal_list")
        self.addDockWidget(Qt.LeftDockWidgetArea, self.dockWidgets["personal"]);
        
    def addActions(self,allActions=True,dockWidgetName=""):
        self.unplugActionList("dockwidget_list")
        if allActions:
            for k,v in self.dockWidgets.iteritems():
                self.myDockActions.append(v.toggleViewAction())
                self.actionCollection().addAction(v.objectName(),v.toggleViewAction())
        else:
            self.myDockActions.append(self.dockWidgets[dockWidgetName].toggleViewAction())
            self.actionCollection().addAction(self.dockWidgets[dockWidgetName].objectName(),self.dockWidgets[dockWidgetName].toggleViewAction())
        self.plugActionList("dockwidget_list",self.myDockActions)
        
    def createDockBugList(self,title,objectname,personal=True,username=""):
        dockWidget=MyBugListDocker(self,title)
        dockWidget.setObjectName(objectname)
        dockWidget.setAllowedAreas(Qt.LeftDockWidgetArea | Qt.RightDockWidgetArea);
        if not personal:
            tabPersonal=tabPersonalBugs(dockWidget,personal,username)
        else:
            tabPersonal=tabPersonalBugs(dockWidget)
        dockWidget.setWidget(tabPersonal)
        dockWidget.setupSlots()
        self.connect(dockWidget,SIGNAL("LPBugRequest(int)"),self.slotGetBugReport)
        self.connect(dockWidget,SIGNAL("fetchBugsInProgress"),self.slotFetchBugsCount)
        return dockWidget
    def slotGetBugReport(self,bugno):
        self.addBugInfoWindow(bugno)
        # self.mainBugInfo.bugRequested(bugno)
    def slotFetchBugsCount(self,in_progress,max_counter,counter):
        logging.getLogger("leonov.kde.views.mainwindow.MainWindow").debug("slotFetchBugsCount: in_progress: %d max_| max_counter: %d | counter: %d",in_progress,max_counter,counter)
        if in_progress==1:
            self.fetch_bugs.setEnabled(False)
            self.progressBar.setMinimum(0)
            self.progressBar.setMaximum(4)
            self.progressBar.hide()
            self.progressBar.setValue(1)
            self.progressBar.show()
            self.statusBar().changeItem(i18n("Fetching Tasklist"),1)
        elif in_progress==0:
            self.fetch_bugs.setEnabled(True)
            self.progressBar.setMaximum(4)
            self.progressBar.setValue(0)
            self.statusBar().changeItem(i18n("Fetching Tasklist finished"),1)
        elif in_progress==2:
            self.progressBar.hide()
            self.progressBar.setValue(max_counter-counter+1)
            self.progressBar.show()
            pass
    def addBugInfoWindow(self,bugno):
        logging.getLogger("leonov.kde.views.mainwindow").debug("addBuginfoWindow")
        if not self.bugWidgets.has_key(str(bugno)):
            logging.getLogger("leonov.kde.views.mainwindow").debug("addBuginfoWindow2")
            self.bugWidgets[str(bugno)]=mainBugInfoWidget(title=i18n("Task Report No #%1",str(bugno)))
            self.connect(self.bugWidgets[str(bugno)],SIGNAL("BugInfoVisitTasksOf"),self.slotVisitTaskOf)
            self.bugWidgets[str(bugno)].bugRequested(bugno)
            self.mainTaskView.addTask(self.bugWidgets[str(bugno)])
        else:
            self.mainTaskView.setActiveTab(self.bugWidgets[str(bugno)])
        
    def setupSlots(self):
        # self.connect(self.tabPersonal.bugModel.getLpBugs(),SIGNAL("LPBugsRetrieveBugNo(int,int,int)"),self.updateProgressBar)
        #self.connect(self.mainBugInfo,SIGNAL("BugInfoVisitTasksOf"),self.slotVisitTaskOf)
        self.connect(self.mainTaskView,SIGNAL("tabClosing"),self.slotTaskClosing)
        pass
    def slotVisitTaskOf(self,bugreporter):
        logging.getLogger("leonov.kde.views.MainWindow").debug("slotVisitTaskOf:")
        if not self.dockWidgets.has_key(str(bugreporter)):
            if type(bugreporter)=="launchpadbugs.lphelper.user":
                self.dockWidgets[str(bugreporter)]=self.createDockBugList(i18n("Tasklist of ")+bugreporter.realname,"dock_"+str(bugreporter),False,str(bugreporter))
            else:
                self.dockWidgets[str(bugreporter)]=self.createDockBugList(i18n("Tasklist of ")+bugreporter,"dock_"+str(bugreporter),False,str(bugreporter))
            self.addDockWidget(Qt.LeftDockWidgetArea, self.dockWidgets[str(bugreporter)]);
            self.addActions(False,str(bugreporter))
            self.dockWidgets[str(bugreporter)].fetchBugs()
    def slotTaskClosing(self,title):
        bugWidgetKey=self.findTaskNo.match(str(title)).group(1)
        del self.bugWidgets[bugWidgetKey]
        
        pass
    def updateStatusBar(self, string_item,itemno):
        
        self.statusBar().changeItem(string_item, itemno)
        
        
    def updateProgressBar(self, counter, block_size, size):
        if self.progressBar.maximum()==1:
            self.progressBar.setMaximum((size/block_size)+1)
        self.progressBar.setValue(counter)

    def slotLaunchpadNewProjectReport(self):
        self.newProjectReport=windowProjectNewReport()
        self.newProjectReport.show()